import sys
import os
from pathlib import Path

from filewiz.file_mover import FileMover
from filewiz.directory_scanner import DirectoryScanner

DEFAULT_ROOT_DIR = "~/Dropbox/accounts/"
DEFAULT_SOURCE_DIR = "~/Desktop/"


def main():
    source_dir_str = sys.argv[1]
    if 'FILEWIZ_TARGET_ROOT' in os.environ:
        root_dir = os.environ['FILEWIZ_TARGET_ROOT']
    else:
        root_dir = DEFAULT_ROOT_DIR
    scanner = DirectoryScanner(source_dir_str)
    mover = FileMover(root_dir)
    scanner.loop_directory(mover)



import time
from pathlib import Path

from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler

from filewiz.file_mover import FileMover


class DirectoryScanner:

    def __init__(self, source_dir_path_str: str):
        self.source_dir_path = Path(source_dir_path_str).expanduser()

    def loop_directory(self, mover: FileMover):
        for file_path in self.source_dir_path.iterdir():
            if not file_path.name.startswith('.'):
                mover.handle_file(file_path)


class NewFileHandler(FileSystemEventHandler):
    def on_created(self, event):
        if not event.is_directory:
            print(event.src_path)
            # move_file(event.src_path, self.target_root)



from pathlib import Path
import re
import sys
from dataclasses import dataclass
from subprocess import run

from wizlib.rlinput import rlinput

PATTERN = re.compile(r'\d{8}\-([a-zA-Z0-9-]+)\.(\w+)')


class FileMover:

    def __init__(self, target_root):
        if isinstance(target_root, str):
            self.target_root_path = Path(target_root).expanduser()
        elif isinstance(target_root, Path):
            self.target_root_path = target_root.expanduser()
        else:
            raise RuntimeError(f"Invalid target root {target_root}")

    def get_parts(self, sub: str) -> set:
        """Return a set of past filename parts"""
        parts = set()
        for year in self.target_root_path.iterdir():
            if year.name.isdigit():
                subdir = year / sub
                if subdir.is_dir():
                    for file in subdir.iterdir():
                        match = re.match(PATTERN, file.name)
                        if match:
                            parts.add(match.groups()[0])
        return parts

    def handle_file(self, source_file_path: Path):
        print()
        assert source_file_path.is_file()
        confirm1 = rlinput(f"Open {source_file_path}? ", default="yes")
        if confirm1.startswith('y'):
            run(['open', source_file_path])
        confirm2 = rlinput(f"Move {source_file_path}? ", default="yes")
        if confirm2.startswith('y'):
            self.move_file(source_file_path)

    def move_file(self, source_file_path: Path):
        """Ask the user some questions then move the file"""
        extension = source_file_path.suffix
        date = input("Date: ")
        account = input("Account: ")
        parts = self.get_parts(account)
        part = rlinput("Part: ", options=parts)
        year = date[:4]
        dirpath = self.target_root_path / year / account
        if not dirpath.exists():
            confirm = rlinput(f"Create {dirpath}? ", default="yes")
            if confirm.startswith('y'):
                dirpath.mkdir(parents=True)
        targetpath = dirpath / f"{date}-{part}{extension}"
        if targetpath.exists():
            print(f"File exists at {targetpath}")
        else:
            confirm = rlinput(f"Move file to {targetpath}? ", default="yes")
            if confirm.startswith('y'):
                source_file_path.rename(targetpath)



import sys
import logging
from watchdog.observers import Observer
from watchdog.events import LoggingEventHandler
from watchdog.events import FileSystemEventHandler


class NewFileHandler(FileSystemEventHandler):
    def on_created(self, event):
        if not event.is_directory:
            print(event.src_path)
        #     # move_file(event.src_path, self.target_root)


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO,
                        format='%(asctime)s - %(message)s',
                        datefmt='%Y-%m-%d %H:%M:%S')
    path = sys.argv[1] if len(sys.argv) > 1 else '.'
    # event_handler = LoggingEventHandler()
    event_handler = NewFileHandler()
    observer = Observer()
    observer.schedule(event_handler, path, recursive=False)
    observer.start()
    try:
        while observer.is_alive():
            observer.join(1)
    finally:
        observer.stop()
        observer.join()
